%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                    %
%                                                                    %
%                                                                    %
%                                                                    %
%                                                                    %
%                                                                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [elemJ, S] = getelemJ(wmri, umri, NE, NI, NS, NDE, NDI, iele, DomNodeID, ASSMtrx, iuldof, ivldof, ipldof, X, Y, ro, mi, C)

elemJ = zeros(21, 21);
S = zeros(21, 21);

NGP = 3;
WGP = [0.555, 0.888, 0.555];
XGP = [-0.7745966, 0.0, 0.7745966];

mascara = zeros(1, 9);
Mri = zeros(2, 9);
XY = zeros(2, 9);
velocity = zeros(2, 9);
pressure = zeros(1, 3);

for ilnode = 1:9
    ignode = DomNodeID(ilnode, iele);
    XY(1, ilnode) = X(ignode);
    XY(2, ilnode) = Y(ignode);
    if ilnode <= 4
        j = X(ignode) + 0.5;
        i = -2*Y(ignode) + 60.5;
        Mri(1, ilnode) = wmri(i, j);
        Mri(2, ilnode) = umri(i, j);
        mascara(1, ilnode) = 1.0;
    else
        Mri(1, ilnode) = 0.0;
        Mri(2, ilnode) = 0.0;
        mascara(1, ilnode) = 0.0;
    end
end

for ilnode = 1:9
    velocity(1, ilnode) = C(ASSMtrx(ilnode, iele));
    velocity(2, ilnode) = C(ASSMtrx(ilnode+9, iele));
end

for ipress = 1:3
    pressure(ipress) = C(ASSMtrx(ipress+18, iele));
end

for igp = 1:NGP
    XI = XGP(igp);
    WI = WGP(igp);
    for jgp = 1:NGP
        NETA = XGP(jgp);
        WJ = WGP(jgp);
        W = WI*WJ;
        [Phi, GradPhi, Psi] = basifunc (XI, NETA);
        
        JAC = GradPhi*XY';
        JACinv = inv(JAC);
        JACdet = det(JAC);
        GradPhixy = JACinv*GradPhi;
        
        UV = velocity*Phi;
        Pre = pressure*Psi;
        
        dUV = velocity*GradPhixy';
        
        for ilnode = 1:9
            iu = iuldof(ilnode);
            iv = ivldof(ilnode);
            for jlnode = 1:9
                ju = iuldof(jlnode);
                jv = ivldof(jlnode);
                
                elemJ(iu,ju) = elemJ(iu,ju) + W*JACdet *(ro* Phi(ilnode)* ...
                    (Phi(jlnode)*dUV(1,1) + UV(1)*GradPhixy(1,jlnode)+ ...
                    UV(2)*GradPhixy(2,jlnode))+...
                    ( 2*mi*GradPhixy(1,ilnode)*GradPhixy(1,jlnode)+ ...
                    mi*GradPhixy(2,ilnode)*GradPhixy(2,jlnode)));%+ ...
                if S(iu,iu) == 0
                    S(iu,iu) = S(iu,iu)+mascara(1,ilnode);
                end
                
                elemJ(iu,jv) = elemJ(iu,jv) + W*JACdet *(ro* Phi(ilnode)* ...
                    Phi(jlnode)*dUV(1,2)+ mi*GradPhixy(2,ilnode)*GradPhixy(1,jlnode));
                
                elemJ(iv,ju) = elemJ(iv,ju) + W*JACdet *(ro* Phi(ilnode)* ...
                    Phi(jlnode)*dUV(2,1) + mi*GradPhixy(1,ilnode)*GradPhixy(2,jlnode));
                
                elemJ(iv,jv) = elemJ(iv,jv) + W*JACdet *(ro* Phi(ilnode)* ...
                    (Phi(jlnode)*dUV(2,2) + UV(1)*GradPhixy(1,jlnode)+ ...
                    UV(2)*GradPhixy(2,jlnode)) + ( 2*mi*GradPhixy(2,ilnode)* ...
                    GradPhixy(2,jlnode)+ mi*GradPhixy(1,ilnode)*GradPhixy(1,jlnode)));%+ ...
                
                if S(iv,iv) == 0
                    S(iv,iv) = S(iv,iv) + mascara(1,ilnode);
                end
            end
            
            for jpress = 1:3
                jp = ipldof(jpress);
                elemJ(iu,jp) = elemJ(iu,jp) + W*JACdet * ...
                    (-GradPhixy(1,ilnode)*Psi(jpress) );
                elemJ(iv,jp) = elemJ(iv,jp) + W*JACdet * ...
                    (-GradPhixy(2,ilnode)*Psi(jpress) );
            end
        end
        
        
        for ipress = 1:3
            ip = ipldof(ipress);
            for jlnode = 1:9
                ju = iuldof(jlnode);
                jv = ivldof(jlnode);
                elemJ(ip,ju) = elemJ(ip,ju)+ W*JACdet*...
                    (GradPhixy(1,jlnode)*Psi(ipress));
                elemJ(ip,jv) = elemJ(ip,jv)+ W*JACdet*...
                    (GradPhixy(2,jlnode)*Psi(ipress));
            end
        end
        
    end
    
end

% Boundary Conditions
% Upper boundary
for i = 1:size(NS,1)
    for ilnode = 1:9
        ignode = DomNodeID(ilnode, iele);
        if ignode == NS(i,1)
            iu = iuldof(ilnode);
            iv = ivldof(ilnode);
            for j = 1:21
                elemJ(iu,j) = 0.0;
                elemJ(iv,j) = 0.0;
            end
            elemJ(iu,iu) = 1.0;
            elemJ(iv,iv) = 1.0;
        end
    end
end

% Bottom boundary
for i = 1:size(NI,1)
    for ilnode = 1:9
        ignode = DomNodeID(ilnode, iele);
        if ignode == NI(i,1)
            iu = iuldof(ilnode);
            iv = ivldof(ilnode);
            for j = 1:21
                elemJ(iu,j) = 0.0;
                elemJ(iv,j) = 0.0;
            end
            elemJ(iu,iu) = 1.0;
            elemJ(iv,iv) = 1.0;
        end
    end
end

% Right upper boundary
for i = 1:size(NDE,1)
    for ilnode = 1:9
        ignode = DomNodeID(ilnode, iele);
        if ignode == NDE(i,1)
            iu = iuldof(ilnode);
            iv = ivldof(ilnode);
        end
    end
end

% Right botoom boundary
for i = 1:size(NDI,1)
    for ilnode = 1:9
        ignode = DomNodeID(ilnode, iele);
        if ignode == NDI(i,1)
            iu = iuldof(ilnode);
            iv = ivldof(ilnode);
            for j = 1:21
                elemJ(iu,j) = 0.0;
                elemJ(iv,j) = 0.0;
            end
            elemJ(iu,iu) = 1.0;
            elemJ(iv,iv) = 1.0;
        end
    end
end

% Left boundary
for i = 1:size(NE,1)
    for ilnode = 1:9
        ignode = DomNodeID(ilnode, iele);
        if ignode == NE(i,1)
            iu = iuldof(ilnode);
            iv = ivldof(ilnode);
            for j = 1:21
                elemJ(iu,j) = 0.0;
                elemJ(iv,j) = 0.0;
            end
            elemJ(iu,iu) = 1.0;
            elemJ(iv,iv) = 1.0;
        end
    end
end

% Setting the pressure level at the external wall
for i = 1:size(NDE,1)
    for ilnode = 1:9
        ignode = DomNodeID(ilnode, iele);
        if ignode == NDE(i,1)
            ip = ipldof(1);
            for j = 1:21
                elemJ(ip,j) = 0.0;
            end
            elemJ(ip,ip) = 1.0;
        end
    end
end

end
