%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                    %
%                                                                    %
%                                                                    %
%                                                                    %
%                                                                    %
%                                                                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [elemRv, b] = getelemRv(wmri, umri, velocidadeEntrada, NE, NI, NS, NDE, NDI, iele, DomNodeID, ASSMtrx, iuldof, ivldof, ipldof, X, Y, ro, mi, C)

    elemRv = zeros(21,1);
    b = zeros(21,1);

    NGP = 3;
    WGP = [0.555, 0.888, 0.555];
    XGP = [-0.7745966, 0.0, 0.7745966]; 

    mascara = zeros(1, 9);
    Mri = zeros(2, 9);
    XY = zeros(2, 9);
    velocity = zeros(2, 9);
    pressure = zeros(1, 3);
        
    for ilnode = 1:9
        ignode = DomNodeID(ilnode, iele);
        XY(1, ilnode) = X(ignode);
        XY(2, ilnode) = Y(ignode);
        if ilnode <= 4
            j = X(ignode) + 0.5;
            i = -2*Y(ignode) + 60.5;
            Mri(1, ilnode) = wmri(i, j);
            Mri(2, ilnode) = umri(i, j);
            mascara(1, ilnode) = 1.0;
        else
            Mri(1, ilnode) = 0.0;
            Mri(2, ilnode) = 0.0;
            mascara(1, ilnode) = 0.0;
        end
    end

    for ilnode = 1:9
        velocity(1, ilnode) = C(ASSMtrx(ilnode, iele));
        velocity(2, ilnode) = C(ASSMtrx(ilnode+9, iele));
    end   

    for ipress = 1:3
        pressure(ipress) = C(ASSMtrx(ipress+18, iele));
    end

    for igp = 1:NGP
       XI = XGP(igp);
       WI = WGP(igp);
       for jgp = 1:NGP
          NETA = XGP(jgp);
          WJ = WGP(jgp);
          W = WI*WJ; 
          [Phi, GradPhi, Psi] = basifunc (XI, NETA);

          JAC = GradPhi*XY';
          JACinv = inv(JAC);
          JACdet = det(JAC);
          GradPhixy = JACinv*GradPhi;

          UV = velocity*Phi;
          Pre = pressure*Psi;

          dUV = velocity*GradPhixy';

          for ilnode = 1:9
             iu = iuldof(ilnode);
             iv = ivldof(ilnode);
             elemRv(iu) = elemRv(iu) + W*JACdet *(ro* Phi(ilnode)*...
                (UV(1)*dUV(1,1) + UV(2)*dUV(1,2))+ GradPhixy(1,ilnode)*...
                ( 2*mi*dUV(1,1) - Pre ) + mi*GradPhixy(2,ilnode)*(dUV(1,2) + dUV(2,1)));% + ...
                if b(iu) == 0
                    b(iu) = b(iu)+(mascara(ilnode)*Mri(1,ilnode))'; %Termo da MRI em W
                end
            
             elemRv(iv) = elemRv(iv) + W*JACdet *(ro* Phi(ilnode)*...
                (UV(1)*dUV(2,1) + UV(2)*dUV(2,2)) + GradPhixy(2,ilnode)*...
                ( 2*mi*dUV(2,2) - Pre ) + mi*GradPhixy(1,ilnode)*(dUV(1,2) + dUV(2,1)));% + ...
                if b(iv)==0
                    b(iv) = b(iv) + (mascara(1,ilnode)*Mri(2,ilnode))'; %Termo da MRI em U
                end
          end   

          for ipress = 1:3
             ip = ipldof(ipress);
             elemRv(ip) = elemRv(ip) + W*JACdet*(dUV(1,1) + dUV(2,2))*Psi(ipress);
          end
       end
    end
    
    % Boundary Conditions
    % Upper boundary
    for i = 1:size(NS,1)
        for ilnode = 1:9
            ignode = DomNodeID(ilnode, iele);
            if ignode == NS(i,1)
                iu = iuldof(ilnode);
                iv = ivldof(ilnode);
                elemRv(iu) = ( velocity(1,ilnode) - 0.0);
                elemRv(iv) = ( velocity(2,ilnode) - 0.0);
            end
        end
    end
    
    % Bottom boundary
    for i = 1:size(NI,1)
        for ilnode = 1:9
            ignode = DomNodeID(ilnode, iele);
            if ignode == NI(i,1)
                iu = iuldof(ilnode);
                iv = ivldof(ilnode);
                elemRv(iu) = ( velocity(1,ilnode) - 0.0);
                elemRv(iv) = ( velocity(2,ilnode) - 0.0);
            end
        end
    end
    
    % Inner right boundary
    for i = 1:size(NDI,1)
        for ilnode = 1:9
            ignode = DomNodeID(ilnode, iele);
            if ignode == NDI(i,1)
                iu = iuldof(ilnode);
                iv = ivldof(ilnode);
                elemRv(iu) = ( velocity(1,ilnode) - 0.0);
                elemRv(iv) = ( velocity(2,ilnode) - 0.0);
            end
        end
    end

    % Left boundary  
    for i = 1:size(NE,1)
        for ilnode = 1:9
            ignode = DomNodeID(ilnode, iele);
            if ignode == NE(i,1)
                iu = iuldof(ilnode);
                iv = ivldof(ilnode);
                elemRv(iu) = ( velocity(1,ilnode) - velocidadeEntrada(1,i));
                elemRv(iv) = ( velocity(2,ilnode) - velocidadeEntrada(2,i));
            end
        end
    end

    % Setting the pressure level at the external wall
    for i = 1:size(NDE,1)
        for ilnode = 1:9
            ignode = DomNodeID(ilnode, iele);
            if ignode == NDE(i,1)
                ip = ipldof(1);
                elemRv(ip) = (pressure(1) - 0.0);
            end
        end
    end
    
end