%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                    %
%                                                                    %
%   Esta funcao ultiliza o mapa de magnitude para diferenciar        % 
%   a malha interna e o contorno do ruido obtido pela MRI            %
%                                                                    %
%                                                                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [malhaVoxels, nVoxelsX, nVoxelsY] = limpaRuido(mag)
        
    %Dimensoes dos Voxels (mm)
    dx = 1.0;
    dy = 0.5;
    
    %Dimensoes da Malha (mm)
    nVoxelsX = 36;
    nVoxelsY = 60;
    L = dx*nVoxelsX;        %comprimento
    H = dy*nVoxelsY;        %largura
    
    nVoxels = 1;
    malhaInterna = zeros(nVoxelsY, nVoxelsX);
    %Diferencia o ruido na matriz Magnitude
    for i = 1:nVoxelsY
        for j = 1:nVoxelsX
            if (mag(i,j) > 0.35)
                malhaInterna(i, j) = 1;
                nVoxels = nVoxels + 1;
            else
                malhaInterna(i, j) = 0;
            end
        end
    end
    %Zera voxels do contorno esquerdo e direito
    for i = 1:nVoxelsY
        malhaInterna(i, 1) = 0;
        malhaInterna(i, nVoxelsX) = 0;
    end
    
    %Zera voxels problem?ticos
    malhaInterna(16, 34) = 0;
    malhaInterna( 9, 30) = 0;
    malhaInterna(36, 26) = 0;
    
    %Verifica qual o contorno da malha interna
    %Varreduta Horizontal
    contorno = malhaInterna;
    eContorno = 1.1;
    for i = 1:nVoxelsY
        for j = 1:nVoxelsX-1
            if((contorno(i, j) == 0) && (contorno(i, j+1) == 1))
                contorno(i, j) = eContorno;
            end
            if((contorno(i, j) == 1) && (contorno(i, j+1) == 0))
                contorno(i, j+1) = eContorno;
            end
            if ((j == 1) && (contorno(i,j) == 1))
                contorno(i, j) = eContorno;
            end
            if ((j == nVoxelsX-1) && (contorno(i, j+1) == 1))
                contorno(i, j+1) = eContorno;
            end
        end
    end
    
    %Varreduta Vertical
    for j = 1:nVoxelsX
        for i = 1:nVoxelsY-1
            if((contorno(i, j) == 0) && (contorno(i+1, j) == 1))
                contorno(i, j) = eContorno;
            end
            if((contorno(i, j) == 1) && (contorno(i+1, j) == 0))
                contorno(i+1, j) = eContorno;
            end
        end
    end
    %Adiciona pontos nas bordas externas
    contorno(23,  1) = eContorno;
    contorno(37,  1) = eContorno;
    contorno(56, 36) = eContorno;
    contorno( 4, 36) = eContorno;
    
    %Zera malha interna no contorno
    contorno = contorno - malhaInterna;

    %Cria malha com todos os voxels
    malhaVoxels = contorno + malhaInterna;
    for j = 1:nVoxelsX
        for i = 1:nVoxelsY
            if malhaVoxels(i, j) == 1.1
                malhaVoxels(i, j) = 1;
            end
        end
    end
    
    
end
